#ifndef _G3DVIEWER_PLUGIN_H
#define _G3DVIEWER_PLUGIN_H

#include <stdio.h>

#include <gtk/gtk.h>
#include <GL/gl.h>

#ifndef G3D_MAX_NAME_LEN
#define G3D_MAX_NAME_LEN 1024
#endif

/*****************************************************************************/
/* G3DImage: for height fields & textures                                    */
/*****************************************************************************/

#define G3D_FLAG_IMG_GREYSCALE	(1L << 1)   /* greyscale image... */

typedef struct {
  char *name;                  /** always useful */
  unsigned int width;          /** width of image */
  unsigned int height;         /** height of image */
  unsigned int depth;          /** color depth in bit */
  unsigned int flags;          /** see defines above */
  unsigned char *pixeldata;    /** width * height * depth bytes */
} G3DImage;

/*****************************************************************************/
/* G3DMaterial                                                               */
/*****************************************************************************/

#define G3D_FLAG_MAT_TWOSIDE 	(1L << 0)

typedef struct {
  char *name;
  GLfloat r,g,b,a;
  guint32 flags;
  GLfloat shininess;
  GLfloat specular[4];
  G3DImage *texture;           /** texture map */
} G3DMaterial;

/*****************************************************************************/
/* G3DFace                                                                   */
/*****************************************************************************/

#define G3D_FLAG_FAC_NORMALS	(1L << 0)
#define G3D_FLAG_FAC_TEXMAP     (1L << 1)

typedef struct {
  G3DMaterial *material;       /** has to be non-NULL */
  int num_vertices;            /** number of vertices in face: 3 or more  */
  int *index_vertex;           /** num_vertices * sizeof(int) */
  guint32 flags;               /** see defines above */
  GLfloat *normals;            /** if face has own normals:
                                   num_vertices * 3 * sizeof(GLfloat) */
} G3DFace;
  
/*****************************************************************************/
/* G3DObject                                                                 */
/*****************************************************************************/

typedef struct {
  char        *name;
  GSList      *materials;
  GSList      *faces;
  unsigned int vertex_count;
  GLfloat     *vertices;
  unsigned int tex_nverts;
  GLfloat     *tex_vertices;
  /* some fields to speed up rendering, should not be used by plugins */
  GLfloat      *_normals;
  G3DMaterial **_materials;
  unsigned int  _num_faces;
  unsigned int *_indices;
  unsigned int *_flags;
} G3DObject;

/*****************************************************************************/
/* G3DModel: more than just for the model information                        */
/*****************************************************************************/

#define G3D_FLAG_GL_SPECULAR		(1L << 0)
#define G3D_FLAG_GL_SHININESS		(1L << 1)
#define G3D_FLAG_GL_ALLTWOSIDE		(1L << 2)

typedef struct {
  GSList *lights;
  GSList *objects;
  GSList *materials;
  float quat[4];
  float zoom;
  struct {
    int beginx, beginy;
  } mouse;
  float aspect;
  double bgcolor[4];
  char *filename;
  guint32 glflags;
  /* Widgets */
  struct {
    GtkWidget *window;
    GtkWidget *infowin;
    struct {
      GtkWidget *tree;
    } modeltab;
    struct {
      GtkWidget *label_desc;
      GtkWidget *label_exts;
    } plugintab;
    GtkWidget *glarea;
    GtkWidget *statushbox;
    GtkWidget *progressbar;
    char show_statusbar;
  } interface;
} G3DModel;

/*****************************************************************************/
/* src/plugins.c: may be used in plugins to load images...                   */
/*****************************************************************************/

#define G3D_PLUGIN_ALL				0x00
#define G3D_PLUGIN_IMPORT         	0x01
#define G3D_PLUGIN_EXPORT			0x02          /* unimplemented */
#define G3D_PLUGIN_IMAGE			0x10		  /* will follow soon.. */
#define G3D_PLUGIN_UNKNOWN        	0xFF

typedef struct {
  gchar *name;
  gchar *path;
  gchar *description;
  gint type;
  gchar **extensions;
} G3DPlugin;

int g3d_plugins_load_model(const char *filename, G3DModel *model);
int g3d_plugins_load_image(const char *filename, G3DImage *image);
gchar **g3d_plugins_get_image_extensions(void);
GSList *g3d_plugins_getlist(int type);

/*****************************************************************************/
/* some interface function: may be changed in near future                    */
/*****************************************************************************/

void g3d_interface_update(void);
void g3d_interface_progress_init(G3DModel *model);
void g3d_interface_progress_update(G3DModel *model, gfloat percentage);
void g3d_interface_progress_finish(G3DModel *model);

/*****************************************************************************/
/* libmisc.a                                                                 */
/*****************************************************************************/

/* readfile.c */
gint32 g3d_read_int8(FILE *f);
gint32 g3d_read_int16_be(FILE *f);
gint32 g3d_read_int16_le(FILE *f);
gint32 g3d_read_int32_be(FILE *f);
gint32 g3d_read_int32_le(FILE *f);
GLfloat g3d_read_float_be(FILE *f);
GLfloat g3d_read_float_le(FILE *f);
void g3d_read_dumphex(FILE *f, int nbytes);

/* iff.c */
#define G3D_IFF_MKID(a,b,c,d) ((((guint32)(a))<<24) | \
                               (((guint32)(b))<<16) | \
                               (((guint32)(c))<< 8) | \
                               (((guint32)(d))    ))

FILE *g3d_iff_open(const char *filename, guint32 *id, guint32 *len);
int g3d_iff_readchunk(FILE *f, guint32 *id, guint32 *len);

/* new.c */
G3DMaterial *g3d_new_G3DMaterial(void);

/* vector.c */
int g3d_vector_normal(GLfloat ax, GLfloat ay, GLfloat az,
                      GLfloat bx, GLfloat by, GLfloat bz,
                      GLfloat *nx, GLfloat *ny, GLfloat *nz);
double g3d_vector_angle(GLfloat ax, GLfloat ay, GLfloat az,
                        GLfloat bx, GLfloat by, GLfloat bz);
int g3d_vector_wrong(GLfloat ax, GLfloat ay, GLfloat az,
                     GLfloat bx, GLfloat by, GLfloat bz);
int g3d_vector_unify(GLfloat *nx, GLfloat *ny, GLfloat *nz);
int g3d_vector_transform(GLfloat *x, GLfloat *y, GLfloat *z, GLfloat *matrix);

#endif
